/**
 * MIT License
 *
 * Copyright (c) 2017-present, Elasticsearch BV
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

import ErrorLogging from './error-logging'
import {
  CONFIG_SERVICE,
  TRANSACTION_SERVICE,
  ERROR_LOGGING,
  APM_SERVER
} from '../common/constants'
import { serviceCreators } from '../common/service-factory'

function registerServices() {
  serviceCreators[ERROR_LOGGING] = serviceFactory => {
    const [
      apmServer,
      configService,
      transactionService
    ] = serviceFactory.getService([
      APM_SERVER,
      CONFIG_SERVICE,
      TRANSACTION_SERVICE
    ])
    return new ErrorLogging(apmServer, configService, transactionService)
  }
}

export { registerServices }
